import os, time
import glob
import pandas
import tqdm
from natsort import natsorted


def convertlabels(sortlist,labels,textlabels):
    clabels = []
    try:
        index = labels.index(sortlist)
        clabel = textlabels[index]
        clabels.append(clabel)
    except:
        clabels = []
    return clabels 

def download_media(link, start, duration):
    r = os.system("yt-dlp -f 'b[height<=480]' --downloader ffmpeg --downloader-args \"ffmpeg:-ss %d -t %d\" '%s'" % (start, duration, link))
    assert r == 0

BASE_URI = "https://www.youtube.com/watch?v="

def main():
    l = pandas.read_excel("labels.xlsx")
    number = l.iloc[:,0].tolist()
    labels = l.iloc[:,1].tolist()
    textlabels = l.iloc[:,2].map(lambda x : x.replace(" ", "")).tolist()
    
    l2 = pandas.read_excel("balanced_train_segments.xlsx")
    
    yid = l2.iloc[:,0].tolist()[2:]
    ystart = l2.iloc[:,1].tolist()[2:]
    yend = l2.iloc[:,2].tolist()[2:]
    ylabels = l2.iloc[:,3].tolist()[2:]
    
    
    err = os.path.abspath("errors.txt")
    try:
        with open(err) as infile:
            errors = set(infile.read().splitlines())
    except Exception as error:
        print(error)
        errors = set()
    existing_files = {x[x.rfind("[") + 1:x.rfind("]")] for x in glob.glob("*/**/*.mp4")}
    existing_files = existing_files.union(errors)
    
    default_dir = os.path.join(os.getcwd(), "audiosetdata")
    print(default_dir)
    try:
        os.mkdir(default_dir)
    except:
        pass
    os.chdir(default_dir)
    
    for i in tqdm.trange(len(yid)):
        if yid[i] in existing_files:
            print("Skipping, already downloaded file.")
            continue

        link = BASE_URI + yid[i]
        start = float(ystart[i])
        end = float(yend[i])
        clabels = convertlabels(ylabels[i], labels, textlabels)

        if clabels == []:
            continue

        newdir = os.path.join(default_dir, clabels[0])
        os.chdir(newdir)
        try:
            download_media(link, start, end - start)
        except Exception as error:
            print("No URLs", error)
            errors.add(yid[i])
            with open(err, "w") as outfile:
                outfile.writelines("\n".join(errors))
        os.chdir(default_dir)


if __name__ == "__main__":
    main()